#!/bin/bash

# KissCount installation script for .tar.bz2 package

# Copyright 2010-2012 Grégory Soutadé

# This file is part of KissCount.

# KissCount is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# KissCount is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with KissCount.  If not, see <http://www.gnu.org/licenses/>.

ROOT_DIR=/usr
SHARE_DIR=$DEST_DIR$ROOT_DIR/share/kisscount
LIB_DIR=$DEST_DIR$ROOT_DIR/lib/kisscount
BIN_DIR=$DEST_DIR$ROOT_DIR/bin

case "$1" in
    --install|-i)
	
	if [ -d $SHARE_DIR -o -d $LIB_DIR -o -f $BIN_DIR/kc ] ; then
	    echo "KissCount already installed, exit"
	    exit
	fi

	sudo mkdir -p $SHARE_DIR
	sudo mkdir -p $LIB_DIR
	sudo mkdir -p ~/.local/share/kisscount
	sudo cp -r ressources/* $SHARE_DIR
	sudo cp kc $BIN_DIR
	sudo cp -r lib/* $LIB_DIR
	
	echo "KissCount successfully installed ! ($DEST_DIR$ROOT_DIR)" 
	;;

    --uninstall|-u)

	sudo rm -rf $SHARE_DIR
	sudo rm -rf $LIB_DIR
	sudo rm -f $BIN_DIR/kc

	echo "Private database (~/.local/share/kisscount) not deleted"
	echo
	echo "KissCount successfully uninstalled !"
	;;
    
    --help|-h)

	echo "KissCount install/uninstall script"
	echo "usage : ./install.sh [options]"
	echo ""
	echo "Options :"
	echo "-i | --install : install KissCount (sudo needed)"
	echo "-u | --uninstall : uninstall KissCount (sudo needed)"
	echo "-h | --help : Display help"
	;;

    *)
	echo "Unknown option $1"

	exit 1
	;;
esac