/*
  Copyright 2010-2012 Grégory Soutadé

  This file is part of KissCount.

  KissCount is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  KissCount is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with KissCount.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef EXPORTENGINE_H
#define EXPORTENGINE_H

#include <model/model.hpp>
#include <controller/KissCount.hpp>
#include <model/AccountAmount.hpp>

class KissCount;

class ExportEngine {
public:
    ExportEngine();
    ~ExportEngine();

    // Get supported file extension. Example :
    // "OFX files (*.ofx)"
    virtual QString GetFileExt();

    // ".ofx"
    virtual QString GetShortExt();

    // Handle the file
    virtual bool HandleFile(const QString& path, User* user, Database* db, KissCount* kiss)=0;

    // Save operations (ExportEngin pre fill _accounts, _categories and _accountAmounts)
    virtual bool SaveFile(std::vector<Operation>* operations)=0;

protected:
    QString _path;
    Database* _db;
    User* _user;
    KissCount* _kiss;

    QString _shortExt;
    QString _longExt;

    std::map<int, int> _accounts;
    std::map<int, int> _categories;
    std::map<AccountAmount, int, AccountAmount> _accountAmounts;
};

#endif
