/*
  Copyright 2010-2012 Grégory Soutadé

  This file is part of KissCount.

  KissCount is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  KissCount is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with KissCount.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef IMPORTENGINE_H
#define IMPORTENGINE_H

#include <model/model.hpp>
#include <controller/KissCount.hpp>
#include <model/AccountAmount.hpp>

class KissCount;

class ImportPattern {
public:
    QString pattern;
    int account;
    int category;
} ;

class ImportEngine {
public:
    static QString NULL_IMPORT_PATTERN;

    ImportEngine();
    ~ImportEngine();

    // Get supported file extension. Example :
    // "OFX files (*.ofx)"
    virtual QString GetFileExt();

    virtual QString GetShortExt();

    // Handle the file
    virtual bool HandleFile(const QString& path, User* user, Database* db, KissCount* kiss)=0;

    // Parse the file and return accounts that doesn't match
    virtual void ParseFile(std::vector<Account>& accounts, std::vector<Category>& categories);
    
    // Final Step
    virtual std::vector<Operation>* GetOperations(std::map<int, int>& accounts, std::map<int, int>& categories);

    const std::map<AccountAmount, int, AccountAmount>& GetAccountAmounts();

    void MatchPattern(QString& key, Operation& op);
    int UpdatePattern(int pos);

    static QString RemoveUnused(const QString& s);

protected:
    Database* _db;
    User* _user;
    QString _path;
    KissCount* _kiss;

    QString _shortExt;
    QString _longExt;

    std::map<int, int> _accounts;
    std::map<int, int> _categories;
    std::vector<Account> _unresolvedAccounts;
    std::vector<Category> _unresolvedCategories;
    std::vector<Operation> _operations;
    std::map<int, QString> _descriptions;
    std::map<AccountAmount, int, AccountAmount> _accountAmounts;

    void ApplyPattern(ImportPattern& pattern, Operation& op);
    QString FindPattern(QString& s1, QString& s2);
    void LinkChilds();
};

#endif
