/*
  Copyright 2010-2012 Grégory Soutadé

  This file is part of KissCount.

  KissCount is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  KissCount is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with KissCount.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GRIDACCOUNT_H
#define GRIDACCOUNT_H

#include <list>
#include <vector>
#include <algorithm>
#include <view/AccountPanel.hpp>
#include <model/model.hpp>
#include <controller/KissCount.hpp>

#include <QTableWidget>
#include <QSignalMapper>
#include <QCompleter>

class KissCount;

typedef void (*updateOperationFunc)(Operation* op, void** params);

class GridAccount : public QTableWidget
{
    Q_OBJECT;

public:
    class OperationNotFound {};

    enum {TREE, DESCRIPTION, OP_DATE, DEBIT, CREDIT, CATEGORY, ACCOUNT, OP_DELETE, CHECKED, NUMBER_COLS_OPS};

    GridAccount(KissCount* kiss, QWidget *parent, 
		bool canAddOperation, bool setWeek, bool synchronizeWithDatabase);
    ~GridAccount();

    virtual void ClearGrid();
    void LoadOperations(std::vector<Operation>* operations, int month, int year);
    void InsertOperationWithWeek(User* user, Operation& op, int line, bool fix, int month, int year) ;
    void InsertOperation(User* user, Operation& op, int line, bool fix, int month, int year) ;

    void GetSelectedOperations(std::vector<int>* rows);

    void MassUpdate(std::vector<int>& rows, bool do_childs, updateOperationFunc func, void** params);

    void Group();
    void UnGroup();

    int _fixCosts;
    int _week1, _week2, _week3, _week4, _week5;
    bool _canAddOperation;
    std::vector<Operation> _displayedOperations;

private slots:
    void OnOperationModified(int row, int col);
    void OnMetaClicked(int id);
    void OnCheckClicked(int id);
    void OnDeleteClicked(int id);

private:
    QWidget* _parent;
    KissCount* _kiss;
    bool _displayLines;
    bool _setWeek;
    bool _databaseSynchronization;
    QString* _categories, *_accounts;
    int _nbAccounts;
    std::vector<Operation>* _operations;
    bool _loadOperations;
    int _curMonth, _curYear;
    QSignalMapper _treeSignalMapper, _checkSignalMapper, _deleteSignalMapper;
    bool _inModification;
    QCompleter* _completer;
    int _transfertCompletionIndex;

    void SetWeek(int week, int line);
    void ResetWeeks();
    void ComputeWeeks();

    bool CheckDay(User* user, const Operation& op, int& i);

    bool IsMetaOpened(int id);
    void OpenMeta(const Operation& meta);
    int InsertIntoGrid(Operation& op);
    void DeleteOperation(const Operation& op) throw (OperationNotFound);
    void RemoveRow(const Operation& op, int line, bool deleteOp);
    void CheckOperation(Operation& op, int line, bool check, bool force);
    void UpdateMeta(Operation& op);
    int  RemoveMeta(Operation op, int line, bool removeRoot, bool deleteOp);
    void CheckMeta(Operation& op, int line, bool check);

    Operation& GetOperation(int id) throw(OperationNotFound);
    void UpdateOperation(Operation& op);
    int GetDisplayedRow(int id) throw (OperationNotFound);
};
#endif
